@@ -12,7 +12,7 @@ BUILD_DATE=""
 
 def get_power_status(conn, options):
 	conn.send("show /system1\n")
-	conn.log_expect(options, "EnabledState=(.*)", POWER_TIMEOUT)
+	conn.log_expect(options, "EnabledState=(.*)", int(options["-Y"]))
 
 	status = conn.match.group(1)
 
@@ -32,7 +32,8 @@ def main():
 	device_opt = [  "help", "version", "agent", "quiet", "verbose", "debug",
 			"action", "ipaddr", "login", "passwd", "passwd_script",
 			"secure", "cmd_prompt", "ipport", "login_eol_lf",
-			"separator", "inet4_only", "inet6_only" ]
+			"separator", "inet4_only", "inet6_only",
+			"power_timeout", "shell_timeout", "login_timeout", "power_wait" ]
 
 	atexit.register(atexit_handler)
 	
@@ -40,7 +41,10 @@ def main():
 	if 0 == options.has_key("-c"):
 		options["-c"] = "MP>"
 		
-	show_docs(options)
+	docs = { }
+	docs["shortdesc"] = "Fence agent for HP iLO MP"
+	docs["longdesc"] = ""
+	show_docs(options, docs)
 	
 	conn = fence_login(options)
 	conn.send("SMCLP\n")
@@ -48,7 +52,7 @@ def main():
 	##
 	## Fence operations
 	####
-	fence_action(conn, options, set_power_status, get_power_status)
+	result = fence_action(conn, options, set_power_status, get_power_status)
 
 	try:
 		conn.send("exit\n")
@@ -56,6 +60,8 @@ def main():
 		pass
 	except pexpect.ExceptionPexpect:
 		pass
+	
+	sys.exit(result)
 
 if __name__ == "__main__":
 	main()
