@@ -12,13 +12,10 @@ REDHAT_COPYRIGHT = ""
 BUILD_DATE = "March, 2008"
 #END_VERSION_GENERATION
 
-POWER_TIMEOUT = 20
-SHELL_TIMEOUT = 3
-LOGIN_TIMEOUT = 5
-
 LOG_MODE_VERBOSE = 100
 LOG_MODE_QUIET = 0
 
+EC_GENERIC_ERROR   = 1
 EC_BAD_ARGS        = 2
 EC_LOGIN_DENIED    = 3
 EC_CONNECTION_LOST = 4
@@ -37,11 +34,15 @@ all_opt = {
 		"getopt" : "h",
 		"longopt" : "help",
 		"help" : "-h, --help                     Display this help and exit",
+		"required" : "0",
+		"shortdesc" : "Display help and exit",
 		"order" : 54 },
 	"version" : { 
 		"getopt" : "V",
 		"longopt" : "version",
 		"help" : "-V, --version                  Output version information and exit",
+		"required" : "0",
+		"shortdesc" : "Display version information and exit",
 		"order" : 53 },
 	"quiet"   : {
 		"getopt" : "q",
@@ -58,6 +59,8 @@ all_opt = {
 		"getopt" : "D:",
 		"longopt" : "debug-file", 
 		"help" : "-D, --debug-file=<debugfile>   Debugging to output file",
+		"required" : "0",
+		"shortdesc" : "Write debug information to given file",
 		"order" : 52 },
 	"agent"   : {
 		"getopt" : "",
@@ -71,6 +74,14 @@ all_opt = {
 		"shortdesc" : "Fencing Action",
 		"default" : "reboot",
 		"order" : 1 },
+	"io_fencing" : {
+		"getopt" : "o:",
+		"longopt" : "action",
+		"help" : "-o, --action=<action>          Action: status, enable or disable",
+		"required" : "1",
+		"shortdesc" : "Fencing Action",
+		"default" : "disable",
+		"order" : 1 },
 	"ipaddr" : {
 		"getopt" : "a:",
 		"longopt" : "ip",
@@ -125,34 +136,49 @@ all_opt = {
 		"getopt" : "m:",
 		"longopt" : "module-name",
 		"help" : "-m, --module-name=<module>     DRAC/MC module name",
+		"required" : "0",
+		"shortdesc" : "DRAC/MC module name",
 		"order" : 1 },
 	"drac_version" : {
 		"getopt" : "d:",
 		"longopt" : "drac-version",
 		"help" : "-d, --drac-version=<version>   Force DRAC version to use",
+		"required" : "0",
+		"shortdesc" : "Force DRAC version to use",
 		"order" : 1 },
 	"hmc_version" : {
 		"getopt" : "H:",
 		"longopt" : "hmc-version",
-		"help" : "-H, --hmc-version=<version>   Force HMC version to use: 3, 4 (default)",
+		"help" : "-H, --hmc-version=<version>    Force HMC version to use: 3, 4 (default)",
+		"required" : "0",
+		"shortdesc" : "Force HMC version to use (3 or 4)",
 		"default" : "4", 
 		"order" : 1 },
 	"ribcl" : {
 		"getopt" : "r:",
 		"longopt" : "ribcl-version",
 		"help" : "-r, --ribcl-version=<version>  Force ribcl version to use",
+		"required" : "0",
+		"shortdesc" : "Force ribcl version to use",
 		"order" : 1 },
+	"login_eol_lf" : {
+		"getopt" : "",
+		"help" : "",
+		"order" : 1
+		},
 	"cmd_prompt" : {
 		"getopt" : "c:",
 		"longopt" : "command-prompt",
 		"help" : "-c, --command-prompt=<prompt>  Force command prompt",
+		"shortdesc" : "Force command prompt",
+		"required" : "0",
 		"order" : 1 },
 	"secure" : {
 		"getopt" : "x",
 		"longopt" : "ssh",
 		"help" : "-x, --ssh                      Use ssh connection",
-		"required" : "0",
 		"shortdesc" : "SSH connection",
+		"required" : "0",
 		"order" : 1 },
 	"ssl" : {
 		"getopt" : "z",
@@ -290,7 +316,38 @@ all_opt = {
 		"longopt" : "separator",
 		"help" : "-C, --separator=<char>         Separator for CSV created by 'list' operation",
 		"default" : ",", 
-		"order" : 100 }
+		"required" : "0",
+		"shortdesc" : "Separator for CSV created by operation list",
+		"order" : 100 },
+	"login_timeout" : {
+		"getopt" : "y:",
+		"longopt" : "login-timeout",
+		"help" : "--login-timeout <seconds>      Wait X seconds for cmd prompt after login",
+		"default" : "5", 
+		"order" : 200 },
+	"shell_timeout" : {
+		"getopt" : "Y:",
+		"longopt" : "shell-timeout",
+		"help" : "--shell-timeout <seconds>      Wait X seconds for cmd prompt after issuing command",
+		"default" : "3", 
+		"order" : 200 },
+	"power_timeout" : {
+		"getopt" : "g:",
+		"longopt" : "power-timeout",
+		"help" : "--power-timeout <seconds>      Test X seconds for status change after ON/OFF",
+		"default" : "20", 
+		"order" : 200 },
+	"power_wait" : {
+		"getopt" : "G:",
+		"longopt" : "power-wait",
+		"help" : "--power-wait <seconds>         Wait X seconds after issuing ON/OFF",
+		"default" : "0", 
+		"order" : 200 },
+	"missing_as_off" : {
+		"getopt" : "M",
+		"longopt" : "missing-as-off",
+		"help" : "--missing-as-off               Missing port returns OFF instead of failure",
+		"order" : 200}
 }
 
 class fspawn(pexpect.spawn):
@@ -306,7 +363,7 @@ def atexit_handler():
 		os.close(1)
 	except IOError:
 		sys.stderr.write("%s failed to close standard output\n"%(sys.argv[0]))
-		sys.exit(1)
+		sys.exit(EC_GENERIC_ERROR)
 
 def version(command, release, build_date, copyright_notice):
 	print command, " ", release, " ", build_date
@@ -317,7 +374,7 @@ def fail_usage(message = ""):
 	if len(message) > 0:
 		sys.stderr.write(message+"\n")
 	sys.stderr.write("Please use '-h' for usage\n")
-	sys.exit(EC_BAD_ARGS)
+	sys.exit(EC_GENERIC_ERROR)
 
 def fail(error_code):
 	message = {
@@ -330,7 +387,7 @@ def fail(error_code):
 		EC_STATUS_HMC : "Failed: Either unable to obtaion correct plug status, partition is not available or incorrect HMC version used"
 	}[error_code] + "\n"
 	sys.stderr.write(message)
-	sys.exit(error_code)
+	sys.exit(EC_GENERIC_ERROR)
 
 def usage(avail_opt):
 	global all_opt
@@ -346,14 +403,15 @@ def usage(avail_opt):
 		if len(value["help"]) != 0:
 			print "   " + value["help"]
 
-def metadata(avail_opt):
+def metadata(avail_opt, options, docs):
 	global all_opt
 
 	sorted_list = [ (key, all_opt[key]) for key in avail_opt ]
 	sorted_list.sort(lambda x, y: cmp(x[1]["order"], y[1]["order"]))
 
 	print "<?xml version=\"1.0\" ?>"
-	print "<resource-agent name=\"" + os.path.basename(sys.argv[0]) + "\" >"
+	print "<resource-agent name=\"" + os.path.basename(sys.argv[0]) + "\" shortdesc=\"" + docs["shortdesc"] + "\" >"
+	print "<longdesc>" + docs["longdesc"] + "</longdesc>"
 	print "<parameters>"
 	for option, value in sorted_list:
 		if all_opt[option].has_key("shortdesc"):
@@ -362,14 +420,42 @@ def metadata(avail_opt):
 			default = ""
 			if all_opt[option].has_key("default"):
 				default = "default=\""+all_opt[option]["default"]+"\""
+			elif options.has_key("-" + all_opt[option]["getopt"][:-1]):
+				if options["-" + all_opt[option]["getopt"][:-1]]:
+					try:
+						default = "default=\"" + options["-" + all_opt[option]["getopt"][:-1]] + "\""
+					except TypeError:
+						## @todo/@note: Currently there is no clean way how to handle lists
+						## we can create a string from it but we can't set it on command line
+						default = "default=\"list of values\""
+			elif options.has_key("-" + all_opt[option]["getopt"]):
+				default = "default=\"true\" "
+
+			mixed = all_opt[option]["help"]
+			## split it between option and help text
+			res = re.compile("^(.*--\S+)\s+", re.IGNORECASE | re.S).search(mixed)
+			if (None != res):
+				mixed = res.group(1)
+			mixed = mixed.replace("<", "&lt;").replace(">", "&gt;")
+			print "\t\t<getopt mixed=\"" + mixed + "\" />"
 
 			if all_opt[option]["getopt"].count(":") > 0:
 				print "\t\t<content type=\"string\" "+default+" />"
 			else:
 				print "\t\t<content type=\"boolean\" "+default+" />"
+				
 			print "\t\t<shortdesc lang=\"en\">" + all_opt[option]["shortdesc"] + "</shortdesc>"
 			print "\t</parameter>"
 	print "</parameters>"
+	print "<actions>"
+	print "\t<action name=\"on\" />"
+	print "\t<action name=\"off\" />"
+	print "\t<action name=\"reboot\" />"
+	print "\t<action name=\"status\" />"
+	print "\t<action name=\"list\" />"
+	print "\t<action name=\"monitor\" />"
+	print "\t<action name=\"metadata\" />"	
+	print "</actions>"
 	print "</resource-agent>"
 
 def process_input(avail_opt):
@@ -451,11 +537,14 @@ def process_input(avail_opt):
 				name = "ipaddr"
 			elif name == "modulename":
 				name = "module_name"
+			elif name == "action" and 1 == avail_opt.count("io_fencing"):
+				name = "io_fencing"
 
 			##
 			######
 			if avail_opt.count(name) == 0:
-				fail_usage("Parse error: Unknown option '"+line+"'")
+				sys.stderr.write("Parse error: Ignoring unknown option '"+line+"'\n")
+				continue
 
 			if all_opt[name]["getopt"].endswith(":"):
 				opt["-"+all_opt[name]["getopt"].rstrip(":")] = value
@@ -481,20 +570,10 @@ def check_input(device_opt, opt):
 	else:
 		all_opt["login"]["required"] = "0"
 
-	## Process special options (and exit)
+	## In special cases (show help, metadata or version) we don't need to check anything
 	#####
-	if options.has_key("-h"): 
-		usage(device_opt)
-		sys.exit(0)
-
-	if options.has_key("-o") and options["-o"].lower() == "metadata":
-		metadata(device_opt)
-		sys.exit(0)
-
-	if options.has_key("-V"):
-		print RELEASE_VERSION, BUILD_DATE
-		print REDHAT_COPYRIGHT
-		sys.exit(0)
+	if options.has_key("-h") or options.has_key("-V") or (options.has_key("-o") and options["-o"].lower() == "metadata"):
+		return options;
 
 	## Set default values
 	#####
@@ -511,9 +590,13 @@ def check_input(device_opt, opt):
 	else:
 		options["log"] = LOG_MODE_QUIET
 
-	if 0 == ["on", "off", "reboot", "status", "list", "monitor"].count(options["-o"].lower()):
-		fail_usage("Failed: Unrecognised action '" + options["-o"] + "'")
-
+	if 0 == device_opt.count("io_fencing"):
+		if 0 == ["on", "off", "reboot", "status", "list", "monitor"].count(options["-o"].lower()):
+			fail_usage("Failed: Unrecognised action '" + options["-o"] + "'")
+	else:
+		if 0 == ["enable", "disable", "status", "list", "monitor"].count(options["-o"].lower()):
+			fail_usage("Failed: Unrecognised action '" + options["-o"] + "'")
+		
 	if (0 == options.has_key("-l")) and device_opt.count("login") and (device_opt.count("no_login") == 0):
 		fail_usage("Failed: You have to set login name")
 
@@ -564,14 +647,41 @@ def check_input(device_opt, opt):
 	return options
 	
 def wait_power_status(tn, options, get_power_fn):
-	for dummy in xrange(POWER_TIMEOUT):
+	for dummy in xrange(int(options["-g"])):
 		if get_power_fn(tn, options) != options["-o"]:
 			time.sleep(1)
 		else:
 			return 1
 	return 0
 
+def show_docs(options, docs = None):
+	device_opt = options["device_opt"]
+
+	if docs == None:
+		docs = { }
+		docs["shortdesc"] = "Fence agent"
+		docs["longdesc"] = ""
+	
+	## Process special options (and exit)
+	#####
+	if options.has_key("-h"): 
+		usage(device_opt)
+		sys.exit(0)
+
+	if options.has_key("-o") and options["-o"].lower() == "metadata":
+		metadata(device_opt, options, docs)
+		sys.exit(0)
+
+	if options.has_key("-V"):
+		print RELEASE_VERSION, BUILD_DATE
+		print REDHAT_COPYRIGHT
+		sys.exit(0)
+
 def fence_action(tn, options, set_power_fn, get_power_fn, get_outlet_list = None):
+	result = 0
+	
+	## Process options that manipulate fencing device
+	#####
 	if (options["-o"] == "list") and (0 == options["device_opt"].count("port")) and (0 == options["device_opt"].count("partition")):
 		print "N/A"
 		return
@@ -595,11 +705,18 @@ def fence_action(tn, options, set_power_fn, get_power_fn, get_outlet_list = None
 	if status != "on" and status != "off":  
 		fail(EC_STATUS)
 
+	
+	if options["-o"] == "enable":
+		options["-o"] = "on"
+	if options["-o"] == "disable":
+		options["-o"] = "off"
+
 	if options["-o"] == "on":
 		if status == "on":
 			print "Success: Already ON"
 		else:
 			set_power_fn(tn, options)
+			time.sleep(int(options["-G"]))
 			if wait_power_status(tn, options, get_power_fn):
 				print "Success: Powered ON"
 			else:
@@ -609,6 +726,7 @@ def fence_action(tn, options, set_power_fn, get_power_fn, get_outlet_list = None
 			print "Success: Already OFF"
 		else:
 			set_power_fn(tn, options)
+			time.sleep(int(options["-G"]))
 			if wait_power_status(tn, options, get_power_fn):
 				print "Success: Powered OFF"
 			else:
@@ -617,17 +735,23 @@ def fence_action(tn, options, set_power_fn, get_power_fn, get_outlet_list = None
 		if status != "off":
 			options["-o"] = "off"
 			set_power_fn(tn, options)
+			time.sleep(int(options["-G"]))
 			if wait_power_status(tn, options, get_power_fn) == 0:
 				fail(EC_WAITING_OFF)
 		options["-o"] = "on"
 		set_power_fn(tn, options)
+		time.sleep(int(options["-G"]))
 		if wait_power_status(tn, options, get_power_fn) == 0:
 			sys.stderr.write('Timed out waiting to power ON\n')
 		print "Success: Rebooted"
 	elif options["-o"] == "status":
 		print "Status: " + status.upper()
+		if status.upper() == "OFF":
+			result = 2
 	elif options["-o"] == "monitor":
 		1
+	
+	return result
 
 def fence_login(options):
 	force_ipvx=""
@@ -638,61 +762,90 @@ def fence_login(options):
 	if (options.has_key("-4")):
 		force_ipvx="-4 "
 
+	if (options["device_opt"].count("login_eol_lf")):
+		login_eol = "\n"
+	else:
+		login_eol = "\r\n"
+
 	try:
 		re_login = re.compile("(login: )|(Login Name:  )|(username: )|(User Name :)", re.IGNORECASE)
 		re_pass  = re.compile("password", re.IGNORECASE)
 
 		if options.has_key("-z"):
 			command = '%s %s %s %s' % (SSL_PATH, force_ipvx, options["-a"], options["-u"])
-			conn = fspawn(command)
+			try:
+				conn = fspawn(command)
+			except pexpect.ExceptionPexpect, ex:
+			 	## SSL telnet is part of the fencing package
+			 	sys.stderr.write(str(ex) + "\n")
+			 	sys.exit(EC_GENERIC_ERROR)
 		elif options.has_key("-x") and 0 == options.has_key("-k"):
 			command = '%s %s %s@%s -p %s' % (SSH_PATH, force_ipvx, options["-l"], options["-a"], options["-u"])
 			if options.has_key("ssh_options"):
 				command += ' ' + options["ssh_options"]
-			conn = fspawn(command)
-
+			try:
+				conn = fspawn(command)
+			except pexpect.ExceptionPexpect, ex:
+				sys.stderr.write(str(ex) + "\n")
+				sys.stderr.write("Due to limitations, binary dependencies on fence agents "
+				"are not in the spec file and must be installed separately." + "\n")
+				sys.exit(EC_GENERIC_ERROR)
+				
 			if options.has_key("telnet_over_ssh"):
 				#This is for stupid ssh servers (like ALOM) which behave more like telnet (ignore name and display login prompt)
-				result = conn.log_expect(options, [ re_login, "Are you sure you want to continue connecting (yes/no)?" ], LOGIN_TIMEOUT)
+				result = conn.log_expect(options, [ re_login, "Are you sure you want to continue connecting (yes/no)?" ], int(options["-y"]))
 				if result == 1:
 					conn.sendline("yes") # Host identity confirm
-					conn.log_expect(options, re_login, LOGIN_TIMEOUT)
+					conn.log_expect(options, re_login, int(options["-y"]))
 
 				conn.sendline(options["-l"])
-				conn.log_expect(options, re_pass, LOGIN_TIMEOUT)
+				conn.log_expect(options, re_pass, int(options["-y"]))
 			else:
-				result = conn.log_expect(options, [ "ssword:", "Are you sure you want to continue connecting (yes/no)?" ], LOGIN_TIMEOUT)
+				result = conn.log_expect(options, [ "ssword:", "Are you sure you want to continue connecting (yes/no)?" ], int(options["-y"]))
 				if result == 1:
 					conn.sendline("yes")
-					conn.log_expect(options, "ssword:", LOGIN_TIMEOUT)
+					conn.log_expect(options, "ssword:", int(options["-y"]))
 
 			conn.sendline(options["-p"])
-			conn.log_expect(options, options["-c"], LOGIN_TIMEOUT)
+			conn.log_expect(options, options["-c"], int(options["-y"]))
 		elif options.has_key("-x") and 1 == options.has_key("-k"):
 			command = '%s %s %s@%s -i %s -p %s' % (SSH_PATH, force_ipvx, options["-l"], options["-a"], options["-k"], options["-u"])
 			if options.has_key("ssh_options"):
 				command += ' ' + options["ssh_options"]
-			conn = fspawn(command)
+			try:
+				conn = fspawn(command)
+			except pexpect.ExceptionPexpect, ex:
+				sys.stderr.write(str(ex) + "\n")
+				sys.stderr.write("Due to limitations, binary dependencies on fence agents "
+				"are not in the spec file and must be installed separately." + "\n")
+				sys.exit(EC_GENERIC_ERROR)
 
-			result = conn.log_expect(options, [ options["-c"], "Are you sure you want to continue connecting (yes/no)?", "Enter passphrase for key '"+options["-k"]+"':" ], LOGIN_TIMEOUT)
+			result = conn.log_expect(options, [ options["-c"], "Are you sure you want to continue connecting (yes/no)?", "Enter passphrase for key '"+options["-k"]+"':" ], int(options["-y"]))
 			if result == 1:
 				conn.sendline("yes")
-				conn.log_expect(options, [ options["-c"], "Enter passphrase for key '"+options["-k"]+"':"] , LOGIN_TIMEOUT)
+				conn.log_expect(options, [ options["-c"], "Enter passphrase for key '"+options["-k"]+"':"] , int(options["-y"]))
 			if result != 0:
 				if options.has_key("-p"):
 					conn.sendline(options["-p"])
-					conn.log_expect(options, options["-c"], LOGIN_TIMEOUT)
+					conn.log_expect(options, options["-c"], int(options["-y"]))
 				else:
 					fail_usage("Failed: You have to enter passphrase (-p) for identity file")
 		else:
-			conn = fspawn(TELNET_PATH)
-			conn.send("set binary\n")
-			conn.send("open %s -%s\n"%(options["-a"], options["-u"]))
-			conn.log_expect(options, re_login, LOGIN_TIMEOUT)
-			conn.send(options["-l"]+"\r\n")
-			conn.log_expect(options, re_pass, SHELL_TIMEOUT)
-			conn.send(options["-p"]+"\r\n")
-			conn.log_expect(options, options["-c"], SHELL_TIMEOUT)
+			try:
+				conn = fspawn(TELNET_PATH)
+				conn.send("set binary\n")
+				conn.send("open %s -%s\n"%(options["-a"], options["-u"]))
+			except pexpect.ExceptionPexpect, ex:
+				sys.stderr.write(str(ex) + "\n")
+				sys.stderr.write("Due to limitations, binary dependencies on fence agents "
+				"are not in the spec file and must be installed separately." + "\n")
+				sys.exit(EC_GENERIC_ERROR)
+
+			conn.log_expect(options, re_login, int(options["-y"]))
+			conn.send(options["-l"] + login_eol)
+			conn.log_expect(options, re_pass, int(options["-Y"]))
+			conn.send(options["-p"] + login_eol)
+			conn.log_expect(options, options["-c"], int(options["-Y"]))
 	except pexpect.EOF:
 		fail(EC_LOGIN_DENIED) 
 	except pexpect.TIMEOUT:
